"""."""

from association_studio.score_driver.score_driver import ScoreDriver


def test_next(driver: ScoreDriver) -> None:
    assert driver.metric_index == 0
    driver.next()
    assert driver.metric_index == 1



"""."""

from association_studio.app.back import AssociationStudioBackend


def test_next_metric(be: AssociationStudioBackend) -> None:
    be.next_metric()
    assert be.score_driver.metric_index == 1



"""."""

from typing import Sequence


def get_multi_line(name: str) -> tuple[str] | tuple[str, str]:
    if name == 'mahalanobis-size-modulated':
        return 'mahalanobis', 'size-modulated'
    else:
        return (name,)


def get_labels(names: Sequence[str]) -> list[str]:
    """."""
    labels = []
    for name in names:
        labels.append('\n'.join(get_multi_line(name)))
    return labels



import matplotlib.pyplot as plt


def get_fig_axes() -> tuple[plt.Figure, plt.Axes, plt.Axes]:
    fig, (ax_bts, ax_fig) = plt.subplots(1, 2, figsize=(10, 4.5))
    ax_bts.set_position([0.05, 0.06, 0.3, 0.85])
    ax_fig.set_position([0.5, 0.06, 0.45, 0.85])
    fig.canvas.manager.set_window_title('Association Studio: metrics')
    return fig, ax_fig, ax_bts



"""."""

import numpy as np
import pytest

from kinematic_tracker.association.metric_giou_yaw import MetricGIoUYaw

from association_studio.exp_metrics.iou_like_two_yaw import IouLikeTwoYaw

from ..conftest import get_kf


def test_rotation(driver: IouLikeTwoYaw, ref_drv: MetricGIoUYaw) -> None:
    """."""
    ref_1z = np.array((0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 4.0, 2.0, 1.0)).reshape(1, 10)
    probe90_z = np.array((0.0, 0.0, 0.0, 0.707106781, 0.0, 0.0, 0.707106781, 4.0, 2.0, 1.0))
    probe180_z = np.array((0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 4.0, 2.0, 1.0))
    kf90 = get_kf(probe90_z)
    kf180 = get_kf(probe180_z)
    ref = ref_drv.compute_metric(ref_1z, [kf90])[0, 0]
    probe90 = driver.compute_metric(ref_1z, [kf90])[0, 0]
    probe180 = driver.compute_metric(ref_1z, [kf180])[0, 0]
    assert probe180 == pytest.approx(1.0)
    assert abs(probe90 - ref) < 0.000085



"""."""

from association_studio.score_driver.metric_labels import get_labels, get_multi_line


def test_get_multi_line() -> None:
    assert get_multi_line('any-thing') == ('any-thing',)
    assert get_multi_line('mahalanobis-size-modulated') == ('mahalanobis', 'size-modulated')


def test_get_labels() -> None:
    assert get_labels(['1-2', '1-2-3']) == ['1-2', '1-2-3']



"""."""

import pytest

from association_studio.score_driver.score_driver import ScoreDriver


def test_flip_selection(driver: ScoreDriver) -> None:
    driver.flip_selection('mahalanobis')
    ref = True, True, False, True, True, True, True
    assert driver.metric_selection == pytest.approx(ref)



"""."""

import re

import numpy as np
import pytest

from association_studio.exp_metrics.check_yaw_quat import check_yaw_quat


def test_it() -> None:
    det_1z = np.zeros((10, 10))
    with pytest.raises(ValueError, match=re.escape('Quaternion must be a 1-dimensional array.')):
        check_yaw_quat(det_1z)

    with pytest.raises(ValueError, match=re.escape('Quaternion must be a 4-element array.')):
        check_yaw_quat(det_1z[0])

    with pytest.raises(ValueError, match=re.escape('Quaternion vector should be normalized.')):
        check_yaw_quat(det_1z[0, 3:7])

    det_1z[:, 4] = 1.0
    with pytest.raises(ValueError, match=re.escape('rx, ry are assumed to be zeros.')):
        check_yaw_quat(det_1z[0, 3:7])

    det_1z[:] = 0.0
    det_1z[:, 5] = 1.0
    with pytest.raises(ValueError, match=re.escape('rx, ry are assumed to be zeros.')):
        check_yaw_quat(det_1z[0, 3:7])

    det_1z[:] = 0.0
    det_1z[:, 3] = -0.5
    with pytest.raises(ValueError, match=re.escape('Scalar part should be positive.')):
        check_yaw_quat(det_1z[0, 3:7])



"""."""

import numpy as np

from association_studio.common import FVT


def check_yaw_quat(vec: FVT) -> None:
    if vec.ndim != 1:
        raise ValueError('Quaternion must be a 1-dimensional array.')

    if vec.size != 4:
        raise ValueError('Quaternion must be a 4-element array.')

    if not np.allclose(vec[1:3], 0.0):
        raise ValueError('rx, ry are assumed to be zeros.')

    if vec[0] < 0.0:
        raise ValueError('Scalar part should be positive.')

    if abs(np.linalg.norm(vec) - 1) > 1.0e-6:
        raise ValueError('Quaternion vector should be normalized.')



"""."""

import numpy as np
import pytest

from association_studio.exp_metrics.iou_like_two_yaw import IouLikeTwoYaw


def test_two_yaw_nu_scenes(driver: IouLikeTwoYaw) -> None:
    vec_z = np.array((1.0, 2.0, 3.0, 4.0, 0.0, 0.0, 7.0, 8.0, 9.0, 10.0))
    driver.two_yaw_nu_scenes(vec_z)
    ref = 1.0, 2.0, 3.0, 0.5076923076923079, 0.0, 0.0, -0.8615384615384615, 8.0, 9.0, 10.0
    assert vec_z == pytest.approx(ref)



"""."""

from association_studio.app.front import AssociationFrontend


def test_it(front: AssociationFrontend) -> None:
    front.on_clicked_buttons('mahalanobis')
    ref = [
        'giou-axes-aligned',
        'giou-yaw',
        'mahalanobis-size-modulated',
        'iou-like-dia-mah',
        'iou-like-dia-slater',
        'iou-like-two-yaw',
    ]
    assert front.back.score_driver.get_names() == ref
    assert len(front.bar.patches) == 6



"""."""

import pytest

from association_studio.common import FVT
from association_studio.score_driver.score_driver import ScoreDriver


def test_compute_scores(vec_z: FVT, driver: ScoreDriver) -> None:
    driver.compute_scores(vec_z)
    ref = (
        0.139487179487,
        0.1902341996,
        0.252306998,
        0.728156653,
        0.181316789,
        0.31068506,
        0.31136363257886335,
    )
    assert driver._scores == pytest.approx(ref)
